package DVD;
use strict;
use warnings;

use Time::Local;
use POSIX qw(ceil);
use HTML::Entities;
use HTTP::Date qw(parse_date);
use File::Spec;
use File::Path;

=head1 NAME

DVD - Package handles an object oriented representation of a DVD

=head1 SYNOPSIS

 use DVD;

=head1 DESCRIPTION

This module provides functions

=cut

use base qw(Exporter);

sub new
{
   my ($class, $owner, $id) = @_;

   return bless {
                   owner => $owner,
                   id    => $id,
                }, $class;
}

sub owner
{
   my $self = shift;

   if (@_)
   {
      chomp($self->{owner} = shift);
   }
   else
   {
      return $self->{owner};
   }
}

sub id
{
   my $self = shift;

   if (@_)
   {
      chomp($self->{id} = shift);
   }
   else
   {
      return $self->{id};
   }
}

sub type
{
   my $self = shift;

   if (@_)
   {
      chomp($self->{type} = shift);
   }
   else
   {
      return $self->{type};
   }
}

sub name
{
   my $self = shift;

   if (@_)
   {
      chomp($self->{name} = shift);
   }
   else
   {
      return $self->{name};
   }
}

sub frontimguri
{
   my $self = shift;

   if (@_)
   {
      $self->{frontimguri} = shift;
   }
   else
   {
      return $self->{frontimguri};
   }
}

sub backimguri
{
   my $self = shift;

   if (@_)
   {
      $self->{backimguri} = shift;
   }
   else
   {
      return $self->{backimguri};
   }
}

sub description
{
   my $self = shift;

   if (@_)
   {
      $self->{description} = shift;
   }
   else
   {
      return $self->{description};
   }
}

sub concatdescription
{
   my ($self, $addtodesc) = @_;

   if (defined($self->description))
   {
      $self->description($self->description . $addtodesc);
   }
   else
   {
      $self->description($addtodesc);
   }
}

my $attrmapping = {
                     "video_formats"   => "format",
                     "genre"           => "genres",
                     "language"        => "audio_tracks",
                     "running_time"    => "length",
                     "other_features"  => "features",
                     "release_year"    => "production_year",
                     "actors"          => "starring",
                  };

sub attribute
{
   my ($self, $attribute, $value, $modifiers) = @_;

   $attribute = lc($attribute);
   chomp $attribute;
   $attribute = $attrmapping->{$attribute} if (exists($attrmapping->{$attribute}));
   if (defined($value))
   {
      chomp $value;
      if (defined($modifiers))
      {
         if (exists($modifiers->{"splitby"}))
         {
            foreach (split(/ */, $modifiers->{"splitby"}))
            {
               $value = splitbychar($value, $_);
            }
         }
         if (exists($modifiers->{"cleantext"}))
         {
            if (ref($value) eq "ARRAY")
            {
               foreach (@{$value})
               {
                  $_ = CleanText($_);
               }
            }
            else
            {
               $value = CleanText($value);
            }
         }
      }
      if (exists($modifiers->{"append"}) && exists($self->{$attribute}))
      {
         my (@oldarray, @newarray);

         if (ref($self->{$attribute}) eq "ARRAY")
         {
            @oldarray = @{$self->{$attribute}};
         }
         else
         {
            @oldarray = ($self->{$attribute});
         }
         if (ref($value) eq "ARRAY")
         {
            @newarray = @{$value};
         }
         else
         {
            @newarray = ($value);
         }
         if ($modifiers->{"append"} eq "nodups")
         {
            foreach my $testval (@newarray)
            {
               push(@oldarray, $testval) unless (grep(/^$testval$/, @oldarray));
            }
            $self->{$attribute} = [ @oldarray ];
         }
         else
         {
            $self->{$attribute} = [ @oldarray, @newarray ];
         }
      }
      else
      {
         if (ref($value) || length($value))
         {
            $self->{$attribute} = $value;
         }
         else
         {
            delete $self->{$attribute} if exists($self->{$attribute});
         }
      }
      if (ref($self->{$attribute}) eq "ARRAY" && @{$self->{$attribute}} == 1)
      {
         $self->{$attribute} = $self->{$attribute}->[0];
      }
   }
   else
   {
      my $retval = ($attribute eq "age") ? $self->age : $self->{$attribute};
      if (ref($retval))
      {
         return wantarray ? @{$retval} : join(", ", @{$retval});
      }
      else
      {
         return defined($retval) ? $retval : "";
      }
   }
}

sub imgname
{
   my ($self, $ext) = @_;

   if ($self->present('frontimguri'))
   {
      my @patharray = $self->frontimguri->path_segments;
      if ($patharray[-1] =~ /[.](\w{1,4})$/)
      {
         return $self->id . "." . (defined($ext) ? $ext : $1);
      }
      return $patharray[-1] . "." . (defined($ext) ? $ext : "jpg");

   }
   return $self->id . "." . (defined($ext) ? $ext : "jpg");
}

sub age
{
   my ($self, $modifiers) = @_;
   my $retval;

   if ($self->present('status'))
   {
      my ($sec1,$min1,$hour1,$mday1,$mon1,$year1,$wday1,$yday1,$isdst1) = localtime();
      my $today  = timelocal(0,0,0,$mday1,$mon1,$year1);
      if ($self->attribute('status') =~ /^(\d{1,4})[\/](\d{1,4})[\/](\d{1,4})$/)
      {
          my $status = timelocal(0,0,0,$2,$1-1,$3);
          $retval = ceil(($today-$status)/(60*60*24));
      }
      else
      {
         my ($year2,$month2,$day2,$hour2,$min2,$sec2,$tz) = parse_date($self->attribute('status'));
         if (defined($day2))
         {
            my $status = timelocal(0,0,0,$day2,$month2,$year2);
            $retval = ceil(($today-$status)/(60*60*24));
         }
         else
         {
            $retval = $self->attribute('status') if (!exists($modifiers->{numeric}));
         }
      }
   }
   if (!defined($retval))
   {
      if (defined($modifiers) && exists($modifiers->{numeric}))
      {
         if ($modifiers->{numeric} eq "only")
         {
            return "0";
         }
         return "";
      }
      return "None";
   }
   return $retval;
}

sub present
{
   my ($self, $attribute) = @_;

   $attribute = lc($attribute);
   chomp $attribute;
   $attribute = $attrmapping->{$attribute} if (exists($attrmapping->{$attribute}));
   return exists($self->{$attribute});
}

sub splitbychar
{
   my ($splitref, $splitchar) = @_;

   $splitref = [$splitref] if (ref($splitref) ne "ARRAY");
   my @newarray = ();
   foreach (@{$splitref})
   {
      @newarray = (@newarray, split / *$splitchar */);
   }
   if (@newarray > 1)
   {
      return \@newarray;
   }
   return $newarray[0];
}

sub detach
{
   my $self = shift;

   $self->{owner} = undef;
}

sub getdetails
{
   my $self = shift;

   $self->owner->getdvddetails($self);
}

sub CleanText
{
   my $text = shift;

   $text = HTML::Entities::decode($text);
   for ($text)
   {
      s/\240//g;
      s/[\n\t\r\f]+/ /g;
      s/^\s+//;
      s/\s+$//;
   }
   return $text;
}

sub fullimgname
{
   my ($self, $indir, $ext, $subdirspec) = @_;

   if (defined($subdirspec))
   {
      $subdirspec = File::Spec->catdir($self->owner->shortname, $subdirspec);
   }
   else
   {
      $subdirspec = $self->owner->shortname;
   }
   my $fullpath = File::Spec->catdir( $indir, $subdirspec );
   mkpath($fullpath) unless -e $fullpath;
   return File::Spec->catfile($fullpath, $self->imgname($ext));
}

sub relimgname
{
   my ($self, $ext) = @_;

   return File::Spec->catfile($self->owner->shortname, $self->imgname($ext));
}

sub getimg
{
   my ($self, $indir) = @_;

   $self->owner->getdvdimg($indir, $self) if ($self->present('frontimguri'));
}

sub mirrorimg
{
   my ($self, $indir) = @_;

   $self->owner->mirrordvdimg($indir, $self) if ($self->present('frontimguri'));
}

sub amplifydetails
{
   my ($self, $dvdsrc) = @_;

   while (my ($attrib, $val) = each(%{$dvdsrc}))
   {
      $self->attribute($attrib, $val) if (!$self->owner->dynamicattrib($attrib));
   }
}

1;
