package DVD::DVDProfiler;
use base DVD::DVDSite;
use strict;
use warnings;

use LWP;
use Carp;
use LWP::ConnCache;
use DVD;

=head1 NAME

DVD::DVDProfiler - Access the DVD Profiler web site to perform functions

=head1 SYNOPSIS

 use DVD::DVDProfiler;

=head1 DESCRIPTION

This module provides functions

=cut

use base qw(Exporter);

################################################################################
#
# Common routines, should be defined for any DVD site
#
################################################################################

sub new
{
   my $class = shift;

   my $self = $class->SUPER::new;
   $self = {   %{$self},
               baseuri        => URI->new('http://www.intervocative.com'),
               framespath     => '/dvdcollection.aspx/',
               loginpath      => 'Login.aspx',
               entrypath      => '/onlinecollections/dvd/InBlue/DVD.aspx',
           };

#   push @{ $self->{ua}->requests_redirectable }, 'POST';

   return bless $self, $class;
}

sub login
{
   my ($self, $userid, $password) = @_;

   $userid = $self->{userid} unless defined($userid);
   croak "Must specify at least an userid to login" unless defined($userid);
   $password = $self->{password} unless defined($password);

   my $response = $self->httpget('loginpath');
   croak "Error getting Login page from DVD Profiler Site: " . $response->status_line unless $response->is_success;

   my $p = GetDVDProfilerVIEWSTATE->new($self);
   $p->parse($response->content);
   croak "Unable to get VIEWSTATE from login page on DVD Profiler Site\n" unless defined($self->{viewstate});

   $response = $self->httppost('loginpath',
                                 [
                                   '__VIEWSTATE'  => $self->{viewstate},
                                   'ebxUsername'  => $userid,
                                   'ebxPassword'  => $password,
                                   'btnSignIn.x'  => '44',
                                   'btnSignIn.y'  => '8',
                                 ]);
   croak "Error Logging in to DVD Profiler Site: " . $response->status_line unless $response->is_success || $response->is_redirect;

   if ($response->is_redirect)
   {
      $response = $self->httpget($response->header('Location'));
      croak "Error Logging in to DVD Profiler Site: " . $response->status_line unless $response->is_success;
   }

   $response = $self->httpget($self->{framespath} . $userid);
   croak "Error Accessing DVD Profiler Site: " . $response->status_line unless $response->is_success;

   $p = GetDVDProfilerURLs->new($self);
   $p->parse($response->content);
   croak "Unable to get frame URL's from base DVD Profiler Site (usually failed login)\n" unless defined($self->{listuri});

   $self->userid($userid);
   $self->password($password);

   $self->getbytype();
}

sub getlist
{
   my $self = shift;

   croak "Can't call getlist until successfully complete login" unless exists($self->{listuri});

   $self->deletelist();

   $self->getbytype('w', 'wishlist');
   $self->getbytype('p', 'ordered');
   $self->getbytype('o', 'owned');

   return $self->list();
}

sub getdvddetails
{
   my ($self, $dvd) = @_;

   croak "Can't call getdvddetails until successfully complete login" unless exists($self->{loadinguri});

   my $response = $self->httpget($self->{entrypath}, U => $dvd->id);
   croak "Error getting DVD Details: " . $response->status_line unless $response->is_success;

   my $p = GetDVDDetails->new($dvd);
   $p->parse($response->content);
   return $dvd;
}

sub shortname
{
   return "DVDProfiler";
}

sub dynamicattrib
{
   my ($self, $val) = @_;

   my $result = grep(/^$val$/, qw(id owner type colnumber));
   return $result;
}

################################################################################
#
# Private routines, specific to this DVD site
#
################################################################################

sub getbytype
{
   my ($self, $typechar, $typename) = @_;

#   my $response = $self->httpget('framespath', alias => $self->{userid}, type => $typechar);
#   croak "Error Accessing DVD Profiler Site: " . $response->status_line unless $response->is_success;

#   my $response = $self->httpget($self->{listuri}, type => $typechar);
   my $response;
   if (defined($typechar))
   {
      $response = $self->httpget($self->{listuri}, type => $typechar);
   }
   else
   {
      $response = $self->httpget($self->{listuri});
   }
   croak "Error getting DVD List: " . $response->status_line unless $response->is_success;

#   $response = $self->httpget("onlinecollections/dvd/InBlue/Loading.aspx");
#   croak "Error getting DVD List: " . $response->status_line unless $response->is_success;

   if (defined($typename))
   {
      my $p = GetDVDList->new($self, $typename);
      $p->parse($response->content);
   }
}

################################################################################
#
# Parse routines to pull the VIEWSTATE cookie out of the first login page
#
################################################################################

package GetDVDProfilerVIEWSTATE;
use base "HTML::Parser";

sub new
{
   my($class, $output) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   return $self;
}

sub start
{
    my ($self, $tag, $attr, $attrseq, $origtext) = @_;

    if ($tag =~ /^input$/i)
    {
       if ($attr->{'name'} =~ /VIEWSTATE$/)
       {
          $self->{output}->{viewstate} = $attr->{'value'};
       }
    }
}

################################################################################
#
# Parse routines to pull the individual frame URL's out of the main page
#
################################################################################

package GetDVDProfilerURLs;
use base "HTML::Parser";

sub new
{
   my($class, $output) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   return $self;
}

sub start
{
    my ($self, $tag, $attr, $attrseq, $origtext) = @_;

    if ($tag =~ /^frame$/i)
    {
       if ($attr->{'name'} =~ /^list$/i)
       {
          $self->{output}->{listuri} = URI->new_abs($attr->{'src'}, $self->{output}->{baseuri});
       }
       elsif ($attr->{'name'} =~ /^entry$/i)
       {
          $self->{output}->{loadinguri} = URI->new_abs($attr->{'src'}, $self->{output}->{baseuri});
       }
    }
}

################################################################################
#
# Parse routines to pull the DVD text entries out of the list frame
#
################################################################################

package GetDVDList;
use base "HTML::Parser";

use constant
    {
    state_LookForEntryLink              =>  0,
    state_FoundName                     =>  1,
    state_LookForColumnNumber           =>  2,
    state_FoundColumnNumber             =>  3,
    state_Done                          =>  4,
    };

sub new
{
   my($class, $output, $type) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   $self->{founddvd} = state_LookForEntryLink;
   $self->{type} = $type;
   return $self;
}

sub start
{
   my ($self, $tag, $attr, $attrseq, $origtext) = @_;

   if ($self->{founddvd} == state_LookForEntryLink && $tag =~ /^a$/i && $attr->{'href'} =~ /U=(.*?)$/i)
   {
      $self->{newdvd} = DVD->new($self->{output}, $1);
      $self->{newdvd}->type($self->{type});
      $self->{founddvd} = state_FoundName;
   }
   elsif ($self->{founddvd} == state_LookForColumnNumber && $tag =~ /^td$/i)
   {
      $self->{founddvd} = state_FoundColumnNumber;
   }
}

sub text
{
    my ($self, $text) = @_;

    if ($text =~ /\S/)
    {
       if ($self->{founddvd} == state_FoundName)
       {
          $self->{newdvd}->name($text);
          $self->{founddvd} = state_LookForColumnNumber;
       }
       elsif ($self->{founddvd} == state_FoundColumnNumber)
       {
          $self->{newdvd}->attribute('colnumber', $text);
          $self->{founddvd} = state_Done;
       }
    }
}

sub end
{
    my ($self, $tag, $origtext) = @_;

    if ($tag =~ /^tr$/i)
    {
       if ($self->{founddvd})
       {
          $self->{output}->addtolist($self->{newdvd});
          $self->{founddvd} = state_LookForEntryLink;
       }
    }
}

################################################################################
#
# Parse routines to pull details about a specific DVD from the
#
################################################################################


package GetDVDDetails;
use base "HTML::Parser";
use Data::Dump qw(pp);

use constant
    {
    state_Virgin                        =>  0,
    state_StartDVD                      =>  1,
    state_WaitingForPrimaryAttributes   =>  2,
    state_FoundPrimaryAttributeName     =>  3,
    state_LookForPrimaryAttributeValue  =>  4,
    state_FoundPrimaryAttributeValue    =>  5,
    state_FoundDescription              =>  6,
    state_LookForFrontImg               =>  7,
    state_LookForRearImg                =>  8,
    state_WaitingForSecondaryAttributes =>  9,
    state_FoundSecondaryAttributeName   => 10,
    state_FoundSecondaryAttributeValue  => 11,
    state_Done                          => 12,
    };

sub new
{
   my($class, $output) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   $self->{dvdstate} = state_Virgin;
   $self->{dvdcurattrlistname} = "";
   $self->{dvdcurattrlist} = [];
   return $self;
}

sub start
{
    my ($self, $tag, $attr, $attrseq, $origtext) = @_;

    if ($self->{dvdstate} == state_Virgin && $tag =~ /^table$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^styleborder$/i)
    {
       $self->{dvdstate} = state_StartDVD;
    }
    elsif (($self->{dvdstate} == state_StartDVD || $self->{dvdstate} == state_WaitingForPrimaryAttributes || $self->{dvdstate} == state_FoundPrimaryAttributeValue) && $tag =~ /^span$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^f3$/i)
    {
       $self->{dvdstate} = state_FoundPrimaryAttributeName;
    }
    elsif ($self->{dvdstate} == state_LookForPrimaryAttributeValue && $tag =~ /^span$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^f2$/i)
    {
       $self->{dvdstate} = state_FoundPrimaryAttributeValue;
    }
    elsif ($self->{dvdstate} >= state_WaitingForPrimaryAttributes && $self->{dvdstate} < state_FoundDescription && $tag =~ /^table$/i)
    {
       $self->{dvdstate} = state_FoundDescription;
    }
    elsif (($self->{dvdstate} == state_FoundDescription ||  $self->{dvdstate} == state_LookForFrontImg) && $tag =~ /^img$/i)
    {
       $self->{output}->description(DVD::CleanText($self->{output}->description));
       $self->{output}->frontimguri(URI->new_abs($attr->{'src'}, $self->{output}->{owner}->{baseuri}));
       $self->{dvdstate} = state_LookForRearImg;
    }
    elsif ($self->{dvdstate} == state_LookForRearImg && $tag =~ /^img$/i)
    {
       $self->{output}->backimguri(URI->new_abs($attr->{'src'}, $self->{output}->{owner}->{baseuri}));
       $self->{dvdstate} = state_WaitingForSecondaryAttributes;
    }
    elsif ($self->{dvdstate} > state_Virgin && exists($attr->{'class'}) && $attr->{'class'} =~ /^styletitle$/i)
    {
       $self->{dvdstate} = state_FoundSecondaryAttributeName;
    }
    elsif ($self->{dvdstate} == state_FoundSecondaryAttributeName && exists($attr->{'class'}) && $attr->{'class'} =~ /^styledata$/i)
    {
       $self->{dvdstate} = state_FoundSecondaryAttributeValue;
    }
}

sub text
{
   my ($self, $text) = @_;

   if ($text =~ /\S/)
   {
      if ($self->{dvdstate} == state_FoundPrimaryAttributeName)
      {
         chomp($self->{dvdtag} = $text);
         $self->{dvdtag} = substr($self->{dvdtag}, 0, length($self->{dvdtag})-1);
         $self->{dvdtag} =~ s/ /_/g;
         $self->{dvdstate} = state_LookForPrimaryAttributeValue;
      }
      elsif ($self->{dvdstate} == state_FoundPrimaryAttributeValue)
      {
         chomp(my $dvdvalue = $text);
         $self->{output}->attribute($self->{dvdtag}, $dvdvalue, {'cleantext' => 1});
         $self->{dvdstate} = state_WaitingForPrimaryAttributes;
      }
      elsif ($self->{dvdstate} == state_FoundDescription)
      {
         $self->{output}->concatdescription($text);
      }
      elsif ($self->{dvdstate} == state_FoundSecondaryAttributeName)
      {
         if (@{$self->{dvdcurattrlist}})
         {
            $self->{output}->attribute($self->{dvdcurattrlistname}, $self->{dvdcurattrlist}, {'cleantext' => 1});
            $self->{dvdcurattrlist} = [];
         }
         chomp($self->{dvdcurattrlistname} = $text);
         $self->{dvdcurattrlistname} =~ s/ /_/g;
      }
      elsif ($self->{dvdstate} == state_FoundSecondaryAttributeValue)
      {
         chomp($self->{dvdcurattr} = $text);
         push(@{$self->{dvdcurattrlist}}, $self->{dvdcurattr});
      }
   }
   elsif ($self->{dvdstate} == state_FoundDescription)
   {
      $self->{output}->concatdescription($text);
   }
}

sub end
{
   my ($self, $tag, $origtext) = @_;

   if ($self->{dvdstate} > state_Virgin && $self->{dvdstate} < state_LookForFrontImg && $tag =~ /^table$/i)
   {
      $self->{dvdstate} = state_LookForFrontImg;
   }
   if ($self->{dvdstate} > state_Virgin && $tag =~ /^body$/i)
   {
      if (@{$self->{dvdcurattrlist}})
      {
         $self->{output}->attribute($self->{dvdcurattrlistname}, $self->{dvdcurattrlist}, {'cleantext' => 1});
      }
      $self->{output}->description(DVD::CleanText($self->{output}->description)) if defined($self->{output}->description);
      $self->{dvdstate} = state_Done;
   }
}

1;
