package DVD::DVDSite;
use strict;
use warnings;

use LWP;
use Carp;
use LWP::ConnCache;
use DVD;

=head1 NAME

DVD::DVDSite - Base set of routines for a package that accesses a DVD site

=head1 SYNOPSIS

 use DVD::DVDSite;

=head1 DESCRIPTION

This module provides functions

=cut

use base qw(Exporter);

sub new
{
   my($class) = @_;

   my $self = {
                  ua            => LWP::UserAgent->new,
                  dvdlist       => {},
              };

   $self->{ua}->cookie_jar( {} );
   $self->{ua}->conn_cache(LWP::ConnCache->new(total_capacity => 3));

#   $self->{ua}->proxy('http', 'http://localhost:8085/');

   return bless $self, $class;
}

sub userid
{
   my $self = shift;

   if (@_)
   {
      $self->{userid} = shift;
   }
   else
   {
      return $self->{userid};
   }
}

sub sourceimages
{
   my $self = shift;

   if (@_)
   {
      $self->{sourceimages} = shift;
   }
   else
   {
      return $self->{sourceimages};
   }
}

sub password
{
   my $self = shift;

   if (@_)
   {
      $self->{password} = shift;
   }
   else
   {
      return $self->{password};
   }
}

sub targetimages
{
   my $self = shift;

   if (@_)
   {
      $self->{targetimages} = shift;
   }
   else
   {
      return $self->{targetimages};
   }
}

sub deletelist
{
   my $self = shift;

   $self->detachlist;
   $self->list({});
}

sub addtolist
{
   my ($self, $dvd) = @_;

   $self->list()->{$dvd->id} = $dvd;
}

sub detachlist
{
   my $self = shift;

   foreach my $dvd (values(%{$self->list()}))
   {
      $dvd->detach;
   }
   my $list = $self->list;
   $self->list(undef);
   return $list;
}

sub attachlist
{
   my ($self, $list) = @_;

   foreach my $dvd (values(%{$list}))
   {
      $dvd->owner($self);
   }
   $self->list($list);
}

sub list
{
   my $self = shift;

   if (@_)
   {
      $self->{dvdlist} = shift;
   }
   else
   {
      return $self->{dvdlist};
   }
}

sub getalldetails
{
   my $self = shift;

   foreach my $dvd (values(%{$self->list()}))
   {
      $dvd->getdetails;
   }
}

sub httpget
{
   my ($self, $path, @queryforms) = @_;

   my $temp_uri = $self->makereluri($path);
   $temp_uri->query_form(@queryforms) if (@queryforms);
   return $self->{ua}->get($temp_uri->as_string, Referer => $temp_uri->as_string);
}

sub httppost
{
   my ($self, $path, @formvalues) = @_;

   my $temp_uri = $self->makereluri($path);
   return $self->{ua}->post($temp_uri->as_string, @formvalues, Referer => $temp_uri->as_string);
}

sub httpgetfile
{
   my ($self, $path, $file) = @_;

   my $temp_uri = $self->makereluri($path);
   return $self->{ua}->get($temp_uri->as_string, Referer => $temp_uri->as_string, ':content_file' => $file);
}

sub httpmirrorfile
{
   my ($self, $path, $file) = @_;

   my $temp_uri = $self->makereluri($path);
   return $self->{ua}->mirror($temp_uri->as_string, $file);
}

sub makereluri
{
   my ($self, $path) = @_;

   my $temp_uri;
   if (ref($path) =~ /^URI::/)
   {
      $temp_uri = $path;
   }
   else
   {
      $temp_uri = $self->{baseuri}->clone;
      if (defined($path))
      {
         if (exists($self->{$path}))
         {
            $temp_uri->path($self->{$path});
         }
         else
         {
            $temp_uri->path($path);
         }
      }
   }
   return $temp_uri;
}

sub getdvdimg
{
   my ($self, $indir, $dvd) = @_;

   croak "No image link determined for dvd " . $dvd->name unless ($dvd->present('frontimguri'));
   my $response = $self->httpgetfile($dvd->frontimguri, $dvd->fullimgname($indir));
   croak "Error getting image " . $dvd->frontimguri->as_string . " from Netflix Site: " . $response->status_line unless $response->is_success;
}

sub mirrordvdimg
{
   my ($self, $indir, $dvd) = @_;

   croak "No image link determined for dvd " . $dvd->name unless ($dvd->present('frontimguri'));
   my $response = $self->httpmirrorfile($dvd->frontimguri, $dvd->fullimgname($indir));
   croak "Error getting image " . $dvd->frontimguri->as_string . " from Netflix Site: " . $response->status_line unless ($response->code == 304 || $response->is_success);
}

sub getallimgs
{
   my ($self, $indir) = @_;

   foreach my $dvd (values(%{$self->list()}))
   {
      $dvd->getimg($indir) if ($dvd->present('frontimguri'));
   }
}

sub mirrorallimgs
{
   my ($self, $indir) = @_;

   foreach my $dvd (values(%{$self->list()}))
   {
      $dvd->mirrorimg($indir) if ($dvd->present('frontimguri'));
   }
}

1;
