package DVD::Netflix;
use base "DVD::DVDSite";
use strict;
use warnings;

use LWP;
use Carp;
use LWP::ConnCache;
use DVD;

=head1 NAME

DVD::Netflix - Access the Netflix web site to perform functions

=head1 SYNOPSIS

 use DVD::Netflix;

=head1 DESCRIPTION

This module provides functions

=cut

use base qw(Exporter);

################################################################################
#
# Common routines, should be defined for any DVD site
#
################################################################################

sub new
{
   my $class = shift;

   my $self = $class->SUPER::new;
   $self = {   %{$self},
               baseuri        => URI->new('http://www.netflix.com'),
               loginuri       => URI->new('https://www.netflix.com/Login'),
               queuepath      => 'Queue',
               moviepath      => 'MovieDisplay',
               login          => 0,
           };

   push @{ $self->{ua}->requests_redirectable }, 'POST';

   return bless $self, $class;
}

sub login
{
   my ($self, $userid, $password) = @_;

   $self->{login} = 0;
   $userid = $self->{userid} unless defined($userid);
   croak "Must specify a userid to login" unless defined($userid);
   $password = $self->{password} unless defined($password);
   croak "Must specify a password to login" unless defined($password);

   my $response = $self->httppost($self->{loginuri},
                                   [
                                     'nextpage'     => 'http://www.netflix.com/Default',
                                     'email'        => $userid,
                                     'movieid'      => '',
                                     'trkid'        => '',
                                     'password1'    => $password,
                                     'SubmitButton' => 'Click Here to Continue',
                                   ]);
   croak "Error Logging in to Netflix Site: " . $response->status_line unless $response->is_success;

   croak "Unable to login to Netflix Site, password or userid failure" if ($self->{ua}->cookie_jar->as_string =~ /Set-Cookie3: fvp=\"\/Login\"/);

   croak "Netflix Site is temporarily unavailable" if ($response->content =~ /temporarily unavailable because of scheduled maintenance/);

   $self->{login} = 1;
   $self->userid($userid);
   $self->password($password);
}

sub getlist
{
   my $self = shift;

   croak "Can't call getlist until successfully complete login" unless ($self->{login});

   $self->deletelist();

   my $response = $self->httpget('queuepath', hnjr => '2');
   croak "Error getting queue from Netflix Site: " . $response->status_line unless $response->is_success;

   my $p = ParseQueue->new($self);
   $p->parse($response->content);

   return $self->list();
}

sub getdvddetails
{
   my ($self, $dvd) = @_;

   croak "Can't call getdvddetails until successfully complete login" unless ($self->{login});

   my $response = $self->httpget('moviepath', movieid => $dvd->id, 'dmode' => 'DETAILS');
   croak "Error getting details from Netflix Site: " . $response->status_line unless $response->is_success;

   my $p = ParseDetails->new($dvd);
   $p->parse($response->content);

   return $dvd;
}

sub shortname
{
   return "Netflix";
}

sub dynamicattrib
{
   my ($self, $val) = @_;

   my $result = grep(/^$val$/, qw(id owner type status queue_number projected_arrival));
   return $result;
}

################################################################################
#
# Private routines, specific to this DVD site
#
################################################################################



################################################################################
#
# Parse routines to get the DVD list out of a Netflix Queue page
#
################################################################################

package ParseQueue;
use base "HTML::Parser";

use constant
    {
    docstate_Virgin                        =>  "virgin",
    docstate_Done                          =>  "done",
    docstate_LookingForOutDVDs             =>  "out",
    docstate_LookingForQueueDVDs           =>  "queued",
    docstate_LookingForAwaitingDVDs        =>  "awaiting",
    };

use constant
    {
    dvdstate_Virgin                        =>  0,
    dvdstate_FoundName                     =>  1,
    dvdstate_LookForRating                 =>  2,
    dvdstate_FoundRating                   =>  3,
    dvdstate_LookForGenre                  =>  4,
    dvdstate_FoundGenre                    =>  5,
    dvdstate_LookForStatus                 =>  6,
    dvdstate_FoundStatus                   =>  7,
    };

sub new
{
   my($class, $output) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   $self->{docstate} = docstate_Virgin;
   $self->{dvdstate} = dvdstate_Virgin;
   return $self;
}

sub start
{
   my ($self, $tag, $attr, $attrseq, $origtext) = @_;

   if ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^dvd(-)out$/i)
   {
      $self->{docstate} = docstate_LookingForOutDVDs;
      $self->SetDVDStateVirgin;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^dvd(-)queue$/i)
   {
      $self->{docstate} = docstate_LookingForQueueDVDs;
      $self->SetDVDStateVirgin;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^dvd(-)awaiting$/i)
   {
      $self->{docstate} = docstate_LookingForAwaitingDVDs;
      $self->SetDVDStateVirgin;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^content$/i)
   {
      $self->{docstate} = docstate_Done;
      $self->SetDVDStateVirgin;
   }
   elsif ($self->{docstate} ne docstate_Virgin && $self->{docstate} ne docstate_Done)
   {
      if ($tag =~ /^input$/i && defined($attr->{'value'}) && $attr->{'value'} =~ /\d+/)
      {
         $self->{inputvalue} = $attr->{'value'};
      }
      elsif ($tag =~ /^a$/i && defined($attr->{'href'}) && $attr->{'href'} !~ /lnkctr/i && $attr->{'href'} =~ /MovieDisplay.*?movieid=(.+?)($|&)/i)
      {
         $self->{newdvd} = DVD->new($self->{output}, $1);
         $self->{newdvd}->type($self->{docstate});
         if (defined($self->{inputvalue}))
         {
            $self->{newdvd}->attribute('queue_number', $self->{inputvalue});
            delete $self->{inputvalue};
         }
         $self->{dvdstate} = dvdstate_FoundName;
      }
      elsif ($self->{dvdstate} == dvdstate_LookForRating && $tag =~ /^td$/i)
      {
         $self->{dvdstate} = dvdstate_FoundRating;
      }
      elsif ($self->{dvdstate} == dvdstate_LookForGenre && $tag =~ /^td$/i)
      {
         $self->{dvdstate} = dvdstate_FoundGenre;
      }
      elsif ($self->{dvdstate} == dvdstate_LookForStatus && $tag =~ /^td$/i)
      {
         $self->{dvdstate} = dvdstate_FoundStatus;
      }
      elsif ($tag =~ /^tr$/i)
      {
         if (defined($self->{inputvalue}))
         {
            delete $self->{inputvalue};
         }
      }
   }
}

sub text
{
   my ($self, $text) = @_;

   if ($text =~ /\S/ && $text !~ /^</)
   {
      if ($self->{dvdstate} == dvdstate_FoundName)
      {
         $self->{newdvd}->name(HTML::Entities::decode($text));
         $self->{dvdstate} = dvdstate_LookForRating;
      }
      elsif ($self->{dvdstate} == dvdstate_FoundRating)
      {
         $self->{newdvd}->attribute('rating', $text, {'cleantext' => 1});
         $self->{dvdstate} = dvdstate_LookForGenre;
      }
      elsif ($self->{dvdstate} == dvdstate_FoundGenre)
      {
         $self->{newdvd}->attribute('genre', $text, {'splitby' => ',', 'cleantext' => 1});
         $self->{dvdstate} = dvdstate_LookForStatus;
      }
      elsif ($self->{dvdstate} == dvdstate_FoundStatus)
      {
         if ($self->{docstate} eq docstate_LookingForOutDVDs && $text =~ / / && $text =~ /\//)
         {
            my @dates = split / /,$text;
            $self->{newdvd}->attribute('status', $dates[0], {'cleantext' => 1});
            $self->{newdvd}->attribute('projected_arrival', $dates[1], {'cleantext' => 1});
         }
         else
         {
            $self->{newdvd}->attribute('status', $text, {'cleantext' => 1});
         }
         $self->SetDVDStateVirgin;
      }
      elsif ($text =~ /\d+/)
      {
         $self->{inputvalue} = $text;
      }
   }
}

sub SetDVDStateVirgin
{
   my $self = shift;

   if (exists($self->{newdvd}))
   {
      $self->{output}->addtolist($self->{newdvd});
      delete $self->{newdvd};
   }
   $self->{dvdstate} = dvdstate_Virgin;
}

################################################################################
#
# Parse routines to get the DVD details out of a Netflix details page
#
################################################################################

package ParseDetails;
use base "HTML::Parser";

use constant
    {
    state_Virgin                        =>  0,
    state_FoundDescription              =>  1,
    state_FoundImage                    =>  2,
    state_LookingForAttributeName       =>  3,
    state_LookingForAttributeValue      =>  4,
    state_FoundYear                     =>  5,
    state_FoundRatingReason             =>  6,
    state_FoundOtherFeaturesPrimed      =>  7,
    state_FoundOtherFeaturesGo          =>  8,
    state_FoundOtherFeaturesBR          =>  9,
    state_FoundOfficialURL              => 10,
    };

my $otherfeatures = [
                       [ "specialfeatures", "other_features" ],
                       [ "otherfeatures"  , "other_features" ],
                       [ "cast"           , "starring"       ],
                       [ "director"       , "director"       ],
                       [ "length"         , "length"         ],
                       [ "genres"         , "genre"          ],
                       [ "screenformats"  , "format"         ],
                       [ "subtitles"      , "subtitles"      ],
                       [ "language"       , "language"       ],
                       [ "releaseyear"    , "release_year"   ],
                       [ "awards",        , "awards"         ],
                    ];
sub new
{
   my($class, $output) = @_;

   my $self = $class->SUPER::new;
   $self->{output} = $output;
   $self->{state} = state_Virgin;
   return $self;
}

sub start
{
   my ($self, $tag, $attr, $attrseq, $origtext) = @_;

   if ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^synopsis$/i)
   {
      $self->{state} = state_FoundDescription;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^boxshot$/i)
   {
      $self->{state} = state_FoundImage;
   }
   elsif ($self->{state} == state_FoundImage && $tag =~ /^img$/i && exists($attr->{'src'}))
   {
      $self->{output}->frontimguri(URI->new($attr->{'src'}));
      $self->{state} = state_Virgin;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^support$/i)
   {
      $self->{state} = state_LookingForAttributeName;
   }
   elsif ($self->{state} == state_LookingForAttributeValue && $tag =~ /^b$/i)
   {
      if (@{$self->{curattributevalue}} > 0)
      {
         $self->{output}->attribute($self->{curattribute}, $self->{curattributevalue}, {'splitby' => ','});
         $self->{formatold} = 1 if ($self->{curattribute} eq 'format');
         $self->{languageold} = 1 if ($self->{curattribute} eq 'language');
      }
      $self->{state} = state_LookingForAttributeName;
   }
   elsif ($tag =~ /^span$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^year$/i)
   {
      $self->{state} = state_FoundYear;
   }
   elsif ($tag =~ /^div$/i && exists($attr->{'class'}) && $attr->{'class'} =~ /^mpaa$/i)
   {
      $self->{state} = state_FoundRatingReason;
   }
   elsif ($tag =~ /^a$/i && exists($attr->{'name'}))
   {
      if (exists($self->{accumtext}))
      {
         $self->outputattribute() if (length($self->{accumtext}));
         delete $self->{accumtext};
      }
      $self->{state} = state_Virgin;
      foreach my $feature (@{$otherfeatures})
      {
         if ($attr->{'name'} =~ /^$feature->[0]$/i)
         {
            $self->{state} = state_FoundOtherFeaturesPrimed;
            $self->{otherattrname} = $feature->[1];
            $self->{accumtext} = '';
            last;
         }
      }
      if ($self->{state} != state_FoundOtherFeaturesPrimed && $attr->{'name'} =~ /^offic.*?url$/i)
      {
         $self->{state} = state_FoundOfficialURL;
      }
   }
   elsif ($tag =~ /^br$/i && $self->{state} == state_FoundOtherFeaturesGo && exists($self->{accumtext}) && length($self->{accumtext}))
   {
      $self->outputattribute();
      $self->{accumtext} = '';
      $self->{state} = state_FoundOtherFeaturesBR;
   }
   elsif ($tag =~ /^br$/i && $self->{state} == state_FoundOtherFeaturesBR)
   {
      if (exists($self->{accumtext}))
      {
         $self->outputattribute() if (length($self->{accumtext}));
         delete $self->{accumtext};
         $self->{state} = state_Virgin;
      }
   }
   elsif ($self->{state} == state_FoundOfficialURL && $tag =~ /^a$/i && exists($attr->{'href'}))
   {
      $self->{output}->attribute("officialurl", $attr->{'href'});
      $self->{state} = state_Virgin;
   }
}

sub text
{
   my ($self, $text) = @_;

   $text = DVD::CleanText($text);
   if ($text =~ /\S/ && $text !~ /^</)
   {
      if ($self->{state} == state_FoundOtherFeaturesBR)
      {
         $self->{state} = state_FoundOtherFeaturesGo;
      }
      if ($self->{state} == state_FoundDescription)
      {
         $text = " " . $text unless ($text =~ /^[.,;:-]/);
         $self->{output}->concatdescription($text);
      }
      elsif ($self->{state} == state_LookingForAttributeName && $text =~ /^(.+):$/)
      {
         $self->{curattribute} = lc($1);
         $self->{curattributevalue} = [];
         $self->{state} = state_LookingForAttributeValue;
      }
      elsif ($self->{state} == state_LookingForAttributeValue && $text ne "," && $text !~ /^more$/i)
      {
         $text =~ /(.+?),*$/;
         push(@{$self->{curattributevalue}}, $1);
      }
      elsif ($self->{state} == state_FoundYear && $text =~ /^[(](\d+)[)]$/i)
      {
         $self->{output}->attribute('release_year', $1);
         $self->{state} = state_Virgin;
      }
      elsif ($self->{state} == state_FoundRatingReason)
      {
         $self->{output}->attribute('rating_reason', $text);
         $self->{state} = state_Virgin;
      }
      elsif ($self->{state} == state_FoundOtherFeaturesGo && exists($self->{accumtext}))
      {
         $self->{accumtext} = $self->{accumtext} . " " . $text;
      }
   }
}

sub end
{
   my ($self, $tag, $origtext) = @_;

   if ($self->{state} == state_LookingForAttributeValue && $tag =~ /^div$/i)
   {
      if (@{$self->{curattributevalue}} > 0)
      {
         $self->{output}->attribute($self->{curattribute}, $self->{curattributevalue}, {'splitby' => ','});
      }
      $self->{state} = state_Virgin;
   }
   elsif ($self->{state} == state_FoundDescription && $tag =~ /^div$/i)
   {
      $self->{output}->description(DVD::CleanText($self->{output}->description));
      $self->{state} = state_Virgin;
   }
   elsif ($self->{state} == state_FoundOtherFeaturesPrimed && $tag =~ /^span$/i)
   {
      $self->{state} = state_FoundOtherFeaturesGo;
   }
}

sub outputattribute
{
   my $self = shift;

   if (exists($self->{formatold}) && $self->{otherattrname} eq 'format')
   {
      $self->{output}->attribute('format', '');
      delete $self->{formatold};
   }
   if (exists($self->{languageold}) && $self->{otherattrname} eq 'language')
   {
      $self->{output}->attribute('language', '');
      delete $self->{languageold};
   }
   $self->{output}->attribute($self->{otherattrname}, $self->{accumtext}, {'splitby' => ',;', 'append' => 'nodups', 'cleantext' => 1});
}

1;

